--[[
	TODO:
		- Maybe add tooltip to indicate whenever double-clicking is going to do something fancy?
]]

local addonName = ...

local function FindFirstCompletedQuest()
	for i = 1, GetNumActiveQuests() do
		if select(2, GetActiveTitle(i)) then
			return i
		end
	end
	return 1 -- Fall back to selecting first incompleted quest if none are completed
end

local function FindFirstNonRepeatableGossipQuest(...) -- GetGossipAvailableQuests returns title#, level#, isLowLevel#, isDaily#, isRepeatable * GetNumGossipAvailableQuests()
	for i = 1, select("#", ...), 5 do
		local daily, repeatable = select(i + 3, ...)
		if not daily and not repeatable then
			return ceil(i / 5)
		end
	end
	return 1
end

local function FindFirstCompletedGossipQuest(...) -- GetGossipActiveQuests returns title#, level#, isLowLevel#, isCompleted# * GetNumGossipActiveQuests()
	for i = 1, select("#", ...), 4 do
		if select(i + 3, ...) then -- Check if isCompleted is true for the quest
			return ceil(i / 4)
		end
	end
	return 1
end

 -- The functions that fire when the frames are double-clicked
local frameHandlers = {
	[GossipFrame] = function(button) -- This seems to show up a lot more than QuestFrameGreetingPanel, which seems quite odd
		if button == "RightButton" then
			GossipFrameGreetingGoodbyeButton:GetScript("OnClick")()
		elseif GetNumGossipAvailableQuests() > 0 then
			SelectGossipAvailableQuest(FindFirstNonRepeatableGossipQuest(GetGossipAvailableQuests()))
		elseif GetNumGossipActiveQuests() > 0 then
			SelectGossipActiveQuest(FindFirstCompletedGossipQuest(GetGossipActiveQuests()))
		elseif GetNumGossipOptions() > 0 then
			SelectGossipOption(1) -- Should this prefer some specific gossip type?
		end
	end,
	[QuestFrameGreetingPanel] = function(button) -- This seems to be used for questgivers that have repeatable quests.
		if button == "RightButton" then
			QuestFrameGreetingGoodbyeButton:GetScript("OnClick")()
		elseif GetNumAvailableQuests() > 0 then
			SelectAvailableQuest(1) -- Can these indices ever be >= 2? And shlould I check for repeatables here too?
		elseif GetNumActiveQuests() > 0 then
			SelectActiveQuest(FindFirstCompletedQuest())
		end
	end,
	[QuestLogFrame] = function(button) -- Should this also let you track/untrack quests? I personally don't use it much, but could do it with modifier+doubleclick I guess
		if button == "RightButton" then
			local dialog = StaticPopup_FindVisible("ABANDON_QUEST") or StaticPopup_FindVisible("ABANDON_QUEST_WITH_ITEMS")
			if dialog then
				return StaticPopup_OnClick(dialog, 1)
			end
			QuestLogFrameAbandonButton:GetScript("OnClick")()
		else
			QuestLogPushQuest()
		end
	end,
	[QuestFrameDetailPanel] = QuestDetailAcceptButton_OnClick, -- These three get the hide-on-rightclick later on in the OnClick function
	[QuestFrameProgressPanel] = QuestProgressCompleteButton_OnClick,
	[QuestFrameRewardPanel] = QuestRewardCompleteButton_OnClick,
}
frameHandlers[QuestLogDetailFrame] = frameHandlers[QuestLogFrame]

local lastTargetFrame
local lastClickTime = 0
local lastButton = ""
local function OnClick(self, button)
	if not frameHandlers[self] then -- As the Quest***Item* buttons aren't in the frameHandlers table, loop through the handler table to see if any of the frames are visible to figure out what we're supposed to be doing
		for frame in pairs(frameHandlers) do
			if frame:IsVisible() then
				return OnClick(frame, button)
			end
		end
	end
	local now = GetTime()
	if now - lastClickTime < 0.5 and self:IsShown() and lastTargetFrame == self and lastButton == button then
		if button == "RightButton" and (self == QuestFrameDetailPanel or self == QuestFrameProgressPanel or self == QuestFrameRewardPanel) then
			QuestGoodbyeButton_OnClick()
		else
			frameHandlers[self](button)
		end
		lastClickTime, lastTargetFrame, lastButton = 0, self, button
		return
	end
	lastClickTime, lastTargetFrame, lastButton = now, self, button
end

-- Helper for hooking other frames such as quest progress or reward buttons
local function HookFrame(name, handler)
	local f = _G[name]
	f:HookScript("OnClick", OnClick)
	frameHandlers[f] = handler
end

-- First lets hook the frames we got in frameHandlers
for frame in pairs(frameHandlers) do
	frame:HookScript("OnMouseUp", OnClick)
	frame:EnableMouse()
end

for i = 1, 6 do
	local b = _G["QuestProgressItem"..i]
	b:HookScript("OnClick", OnClick)
	b:RegisterForClicks("LeftButtonUp", "RightButtonUp")
end

 -- These buttons are shared between the QuestFrameDetailPanel and QuestFrameRewardPanel, and we don't want the reward type to be double-clickable so we don't accidentally select the wrong rewards when clicking really fast.
local function QuestInfoItemHandler(self, button)
	if button == "RightButton" or not QuestFrameRewardPanel:IsVisible() then
		OnClick(self, button)
	end
end
for i = 1, 10 do
	local b = _G["QuestInfoItem"..i]
	b:HookScript("OnClick", QuestInfoItemHandler)
	b:RegisterForClicks("LeftButtonUp", "RightButtonUp")
end

-- Well this turned out a bit ugly :-(
local function LootFrameHandler()
	local dialog = StaticPopup_FindVisible("CONFIRM_LOOT_ROLL")
	if dialog then
		StaticPopup_OnClick(dialog, 1)
	end
end
for i = 1, 4 do
	HookFrame("GroupLootFrame"..i.."RollButton", LootFrameHandler)
	HookFrame("GroupLootFrame"..i.."GreedButton", LootFrameHandler)
	HookFrame("GroupLootFrame"..i.."DisenchantButton", LootFrameHandler)
end

HookFrame = nil -- This will save like, maybe multiple bytes of memory zomg!

----------------------------------------------------

-- Slashcommand to show version info, 'cos having to open the TOC to check it sucks
_G["SLASH_"..addonName:upper().."1"], _G["SLASH_"..addonName:upper().."2"] = "/"..addonName:lower(), "/"..addonName:lower().."version"
SlashCmdList[addonName:upper()] = function()
	print(addonName, "version:", GetAddOnMetadata(addonName, "Version"))
end
